//
//  AsReaderViewController.m
//  AreteCap
//
//  Created by Asterisk on 2014/06/23.
//
//

#import "AsReaderViewController.h"
#import "RcpBarcodeApi.h"
#import "RcpRfidApi.h"
#import <AudioToolBox/AudioServices.h>


@interface AsReaderViewController ()<RcpBarcodeDelegate,RcpRfidDelegate>
{
    BOOL plugged;
    CFURLRef		soundFileURLRef;
	SystemSoundID	soundFileObject;
    int prev_battery;
}
@property (nonatomic, strong, readwrite) RcpBarcodeApi *rcpBarcode;
@property (nonatomic, strong, readwrite) RcpRfidApi *rcpRfid;
@property (weak, nonatomic) IBOutlet UITextView *myTextView;
@property (weak, nonatomic) IBOutlet UILabel *statusLabel;
- (void)addText:(NSString*)value;
@end

@implementation AsReaderViewController

- (void)appWillTerminate:(NSNotification*) notification{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)appDidEnterBackground:(NSNotification*) notification{
    if([self.rcpBarcode isOpened]){
        [self.rcpBarcode setReaderPower:NO];
        [self.rcpBarcode close];
    }
    
    if([self.rcpRfid isOpened]){
        [self.rcpRfid setReaderPower:NO];
        [self.rcpRfid close];
    }
}

- (void)viewDidLoad{
    [super viewDidLoad];

    _myTextView.font = [UIFont systemFontOfSize:30];
    [[NSNotificationCenter defaultCenter]addObserver:self
                                            selector:@selector(appDidEnterBackground:)
                                                name:UIApplicationDidEnterBackgroundNotification
                                              object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(appWillTerminate:)
                                                 name:UIApplicationWillTerminateNotification
                                               object:nil];
}

- (void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    
    NSURL *tapSound   = [[NSBundle mainBundle] URLForResource: @"read" withExtension: @"caf"];
    // Store the URL as a CFURLRef instance
    soundFileURLRef = (__bridge CFURLRef) tapSound;
    
    // Create a system sound object representing the sound file.
    AudioServicesCreateSystemSoundID (soundFileURLRef,&soundFileObject);
}

- (void)didReceiveMemoryWarning{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)viewWillDisappear:(BOOL)animated{
	[super viewWillDisappear:animated];
    AudioServicesDisposeSystemSoundID (soundFileObject);
    [self.rcpBarcode close];
    [self.rcpRfid close];
}

- (void)viewDidAppear:(BOOL)animated{
    [super viewDidAppear:animated];
    self.rcpBarcode.delegate = self;
    self.rcpRfid.delegate = self;
}

- (RcpBarcodeApi *)rcpBarcode{
    static dispatch_once_t pred = 0;
    __strong static id _sharedObject = nil;
    
    dispatch_once(&pred,^{
        _sharedObject = [[RcpBarcodeApi alloc] init];
        _rcpBarcode = _sharedObject;
        _rcpBarcode.delegate = self;
        [_rcpBarcode open];
    });
    return _sharedObject;
}

- (RcpRfidApi *)rcpRfid{
    static dispatch_once_t pred = 0;
    __strong static id _sharedObject = nil;
    
    dispatch_once(&pred,^{
        _sharedObject = [[RcpRfidApi alloc] init];
        _rcpRfid = _sharedObject;
        _rcpRfid.delegate = self;
        [_rcpRfid open];
    });
    return _sharedObject;
}

- (IBAction)didTapToClear:(id)sender {
    _myTextView.text = @"";
}

#pragma mark RcpDelegate Method
- (void)barcodeReceived:(NSData *)barcode{
    NSLog(@"barcodeReceived");
    dispatch_async(dispatch_get_main_queue(),^{
        AudioServicesPlaySystemSound (soundFileObject);
        
        NSString *scanStr = [[NSString alloc]initWithData:barcode encoding:NSShiftJISStringEncoding];
        
        NSLog(@"scanStr:%@",scanStr);
        [self addText:scanStr];
    });
}

- (void)epcReceived:(NSData *)epc{
    dispatch_async(dispatch_get_main_queue(),^{
        
        NSMutableString* tag = [[NSMutableString alloc] init];
        unsigned char* ptr= (unsigned char*) [epc bytes];
        
        for(int i = 0; i < epc.length; i++) {
            [tag appendFormat:@"%02X", *ptr++ & 0xFF ];
        }
        
        [self addText:tag];
    });
}

- (void)pcEpcRssiReceived:(NSData *)pcEpc rssi:(int8_t)rssi{
    dispatch_async(dispatch_get_main_queue(),^{
        //        AudioServicesPlaySystemSound (soundFileObject);
        
        NSMutableString* tag = [[NSMutableString alloc] init];
        unsigned char* ptr= (unsigned char*) [pcEpc bytes];
        
        for(int i = 0; i < pcEpc.length; i++) {
            [tag appendFormat:@"%02X", *ptr++ & 0xFF ];
        }
        
        [self addText:[NSString stringWithFormat:@"%@::%@",[NSNumber numberWithInt:(rssi)],tag]];
    });
}

- (void)pluggedBarcode:(BOOL)plug{
    plugged = plug;
    
    if(plug){
        if([self.rcpBarcode isOpened]){
            dispatch_async(dispatch_get_main_queue(),^{
                _statusLabel.text = @"Plugged";
                [self.rcpBarcode setReaderPower:YES];
            });
        }
    }else{
        dispatch_async(dispatch_get_main_queue(),^{
            _statusLabel.text = @"Unplugged";
            [self.rcpBarcode setReaderPower:NO];
            [self.rcpBarcode close];
        });
    }
}

-(void)onUpdate{
    dispatch_async(dispatch_get_main_queue(),^{
//        [self.rcpBarcode setReaderPower:YES];
    });
}

- (void)pluggedRfid:(BOOL)plug{
    plugged = plug;
    
    if(plug){
        if([self.rcpRfid isOpened]){
            dispatch_async(dispatch_get_main_queue(),^{
                _statusLabel.text = @"Plugged";
                [self.rcpRfid setReaderPower:YES];
            });
        }
    }else{
        dispatch_async(dispatch_get_main_queue(),^{
            _statusLabel.text = @"Unplugged";
            [self.rcpRfid setReaderPower:NO];
            [self.rcpRfid close];
        });
    }
}

- (void)stoppedReadBarcodes:(uint8_t)statusCode{
    NSLog(@"stoppedReadBarcodes status : %d\n",statusCode);
    dispatch_async(dispatch_get_main_queue(), ^{
        self.statusLabel.text = @"BC STOP";
    });

    switch (statusCode)
    {
        case 0x00:
            NSLog(@"Read Barcodes Stopped\n");
            break;
        default:
            break;
    }
}


- (IBAction)startRead:(id)sender {
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.rcpRfid startReadTags:0x00 mtime:0x00 repeatCycle:0x00];
        [self.rcpBarcode startReadBarcodes:0x00 mtime:0x00 repeatCycle:0x00];
    });
}

- (IBAction)stopRead:(id)sender {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.rcpBarcode stopReadBarcodes];
            [self.rcpRfid stopReadTags];
        });
}

#pragma mark - Private Methodw

- (void)addText:(NSString*)value{
    if ([_myTextView.text isEqualToString:@""] || _myTextView.text == nil) {
        _myTextView.text = value;
    }else{
        _myTextView.text = [NSString stringWithFormat:@"%@\n%@",_myTextView.text,value];
    }
    dispatch_async(dispatch_get_main_queue(), ^{
        
        NSRange range = NSMakeRange(_myTextView.text.length - 1, 1);
        [_myTextView scrollRangeToVisible:range];
        
    });
}


@end
